"""
Represents an VPC Security Group
"""
from footmark.vpc.vpcobject import TaggedVPCObject


class Eip(TaggedVPCObject):
    def __init__(self, connection=None, owner_id=None,
                 name=None, description=None, id=None):
        super(Eip, self).__init__(connection)
        self.tags = {}

    def __repr__(self):
        return 'Eip:%s' % self.id

    def __getattr__(self, name):
        if name in ('id', 'eip_id'):
            return self.allocation_id
        if name in ('ip', 'eip', 'eip_address'):
            return self.ip_address

        raise AttributeError

    def __setattr__(self, name, value):
        if name == 'status':
            value = str.lower(value)
        if name in ('ip', 'eip', 'eip_address'):
            self.ip_address = value
        if name == 'descritpion':
            name = 'description'
        if name == 'tags' and value:
            v = {}
            for tag in value['tag']:
                v[tag.get('key')] = tag.get('value', None)
            value = v
        super(TaggedVPCObject, self).__setattr__(name, value)

    def get(self):
        return self.connection.describe_eip_addresses(allocation_id=self.id)[0]
    
    def associate(self, instance_id):
        """
        bind eip
        """
        if str(self.status).lower() == 'inuse':
            if self.instance_id == instance_id:
                return False
            raise Exception('EIP {0} current status {1} does not support association.'.format(self.id, self.status))
        instance_type = "EcsInstance"
        if str(instance_id).startswith("lb-"):
            instance_type = "SlbInstance"
        if str(instance_id).startswith("nat-"):
            instance_type = "Nat"
        if str(instance_id).startswith("havip-"):
            instance_type = "HaVip"
        if str(instance_id).startswith("eni-"):
            instance_type = "NetworkInterface"

        return self.connection.associate_eip_address(allocation_id=self.id, instance_id=instance_id, instance_type=instance_type)

    def unassociate(self, instance_id):
        """
        unbind eip
        """
        if str(self.status).lower() == 'available':
            return False
        return self.connection.unassociate_eip_address(allocation_id=self.id, instance_id=instance_id)
    
    def release(self):
        """
        release eip
        """
        return self.connection.release_eip_address(allocation_id=self.id)
    
    def modify(self, bandwidth=None, name=None, description=None):
        """
        modify eip
        """
        params = {}
        if bandwidth and int(self.bandwidth) != int(bandwidth):
            params['bandwidth'] = bandwidth
        if name and self.name != name:
            params['name'] = name
        if description and self.description != description:
            params['description'] = description
        if params:
            params['allocation_id'] = self.allocation_id
            return self.connection.modify_eip_address_attribute(**params)
        return False

    def read(self):
        eip = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "available_regions", "operation_locks", "resource_group_id",
                        "has_reservation_data", "hdmonitor_status", "isp"]:
                continue

            if name == 'allocation_id':
                eip['id'] = value

            eip[name] = value
        return eip

    def add_tags(self, tags):
        """
        Add tags
        """
        return self.connection.tag_resources(resource_ids=[self.id], tags=tags, resource_type='eip')

    def remove_tags(self, tags):
        """
        remove tags
        """
        return self.connection.un_tag_resources(resource_ids=[self.id], tags=tags, resource_type='eip')




from footmark.dns.dnsobject import TaggedDNSObject


class Dns(TaggedDNSObject):
    def __init__(self, connection=None):
        super(Dns, self).__init__(connection)

    def __repr__(self):
        return 'Dns:%s' % self.id

    def __getattr__(self, name):
        if name == 'id':
            return self.id
        if name == 'name':
            return self.domain_name
        if name == 'domain_remark':
            if hasattr(self, 'remark'):
                return self.remark
            return ''
        if name == 'group_id':
            if hasattr(self, 'gid'):
                return self.gid
            return ''
        raise AttributeError

    def __setattr__(self, name, value):
        if name == 'domain_id':
            self.id = value
        if name == 'domain_name':
            self.name = value
        if name == 'group_id':
            self.gid = value
        super(TaggedDNSObject, self).__setattr__(name, value)

    def get(self):
        return self.connection.describe_domain_info(domain_name=self.name)

    def read(self):
        dns = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region"]:
                continue

            if name == 'domain_id':
                dns['id'] = value
            dns[name] = value
        return dns

    def delete(self):
        return self.connection.delete_domain(domain_name=self.name)

    def modify_remark(self, remark=None):
        if remark and self.domain_remark != remark:
            return self.connection.update_domain_remark(domain_name=self.name, remark=remark)
        return False

    def change_domain_group(self, group_name=None, domain_name=None):
        group_id = self.connection.describe_domain_group(group_name=group_name).id
        if self.group_id != group_id:
            return self.connection.change_domain_group(group_id=group_id, domain_name=domain_name)



class OOSObject(object):
    def __init__(self, connection=None):
        self.connection = connection
        if self.connection and hasattr(self.connection, 'region'):
            self.region = connection.region
        else:
            self.region = None


class TaggedOOSObject(OOSObject):
    def __init__(self, connection=None):
        super(TaggedOOSObject, self).__init__(connection)



"""
Represents an VPC Security Group
"""
from footmark.ram.ramobject import TaggedRAMObject


class User(TaggedRAMObject):
    def __init__(self, connection=None):
        super(User, self).__init__(connection)

    def __repr__(self):
        return 'Ram:%s' % self.id

    def __getattr__(self, name):
        if name == 'name':
            return self.user_name
        if name == 'mobile_phone':
            return self.phone

    def __setattr__(self, name, value):
        if name == 'user_name':
            self.name = value
        if name == 'user_id':
            self.id = value
        if name == 'mobile_phone':
            self.phone = value
        if name == 'user':
            for k, v in value.items():
                super(TaggedRAMObject, self).__setattr__(k, v)
        super(TaggedRAMObject, self).__setattr__(name, value)

    def read(self):
        ram = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            if name == 'user':
                for k, v in value.items():
                    ram[k] = v
                    setattr(self, k, v)
                continue
            ram[name] = value
        return ram

    def delete(self):
        return self.connection.delete_user(UserName=self.name)

    def update(self, **kwargs):
        params = {}
        if kwargs.get('new_user_name') and kwargs.get('new_user_name') != self.name:
            params['new_user_name'] = kwargs.get('new_user_name')
        if kwargs.get('mobile_phone') and kwargs.get('mobile_phone') != self.mobile_phone:
            params['new_mobile_phone'] = kwargs.get('mobile_phone')
        if kwargs.get('display_name') and kwargs.get('display_name') != self.display_name:
            params['new_display_name'] = kwargs.get('display_name')
        if kwargs.get('email') and kwargs.get('email') != self.email:
            params['new_email'] = kwargs.get('email')
        if kwargs.get('comments') and kwargs.get('comments') != self.comments:
            params['new_comments'] = kwargs.get('comments')
        if params:
            params['user_name'] = self.name
            params['new_user_name'] = kwargs.get('new_user_name') if kwargs.get('new_user_name') else params['user_name']
            return self.connection.update_user(**params)
        return None


class Profile(TaggedRAMObject):
    def __init__(self, connection=None):
        super(Profile, self).__init__(connection)

    def __repr__(self):
        return 'Profile:%s' % self.id

    def __getattr__(self, name):
        if name == 'name':
            return self.user_name

    def __setattr__(self, name, value):
        if name == 'user_name':
            self.name = value
        super(TaggedRAMObject, self).__setattr__(name, value)

    def get(self):
        return self.connection.get_login_profile(user_name=self.user_name)

    def read(self):
        ram = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            if name == 'login_profile':
                for key, value in value.items():
                    ram[key] = value
                continue
            ram[name] = value
        return ram

    def delete(self):
        return self.connection.delete_login_profile(user_name=self.name)

    def update(self, **kwargs):
        if kwargs.get('password_reset_required') != self.login_profile['password_reset_required'] or kwargs.get('mfa_bind_required') != self.login_profile['mfabind_required'] or kwargs.get('new_password'):
            if kwargs.get('new_password'):
                kwargs['password'] = kwargs.get('new_password')
            return self.connection.update_login_profile(**kwargs)
        return False


class Access(TaggedRAMObject):
    def __init__(self, connection=None):
        super(Access, self).__init__(connection)

    def __repr__(self):
        return 'Access:%s' % self.id

    def __getattr__(self, name):
        pass

    def __setattr__(self, name, value):
        super(TaggedRAMObject, self).__setattr__(name, value)

    def read(self):
        access_key = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            if name == 'access_key':
                for key, value in value.items():
                    access_key[key] = value
                continue
            access_key[name] = value
        return access_key


class Group(TaggedRAMObject):
    def __init__(self, connection=None):
        super(Group, self).__init__(connection)

    def __repr__(self):
        return 'Group:%s' % self.id

    def __getattr__(self, name):
        pass

    def __setattr__(self, name, value):
        if name == 'group_name':
            self.name = value
        super(TaggedRAMObject, self).__setattr__(name, value)

    def read(self):
        group = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            group[name] = value
        return group

    def delete(self):
        return self.connection.delete_group(group_name=self.name)

    def update(self, comments=None, new_group_name=None):
        params = {}
        if comments and comments != self.comments:
            params['new_comments'] = comments
        if new_group_name and new_group_name != self.name:
            params['new_group_name'] = new_group_name
        if params:
            params['group_name'] = self.name
            return self.connection.update_group(**params)
        return False

    def add_user(self, user_name=None):
        users = self.connection.list_users_for_group(group_name=self.name)
        flag = False
        for user in users:
            if user.user_name == user_name:
                flag = True
        if not flag:
            return self.connection.add_user_to_group(user_name=user_name, group_name=self.name)
        return False

    def remove_user(self, user_name=None):
        users = self.connection.list_users_for_group(group_name=self.name)
        flag = False
        for user in users:
            if user.user_name == user_name:
                flag = True
        if flag:
            return self.connection.remove_user_from_group(user_name=user_name, group_name=self.name)
        return False


class Role(TaggedRAMObject):
    def __init__(self, connection=None):
        super(Role, self).__init__(connection)

    def __repr__(self):
        return 'Role:%s' % self.id

    def __getattr__(self, name):
        if name == 'name':
            return self.role_name

    def __setattr__(self, name, value):
        if name == 'role_name':
            self.name = value
        super(TaggedRAMObject, self).__setattr__(name, value)

    def read(self):
        role = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            if name == 'role':
                for k, v in value.items():
                    role[k] = v
                continue
            role[name] = value
        return role

    def get(self):
        return self.connection.get_role(role_name=self.name)

    def delete(self):
        return self.connection.delete_role(role_name=self.name)

    def update_policy(self, policy=None):
        params = {}
        role_policy = self.connection.get_role(role_name=self.name).read()['assume_role_policy_document']
        role_policy = role_policy.replace('\n', '').replace(' ', '')
        policy = policy.replace('\n', '').replace(' ', '')
        if policy and policy != role_policy:
            params['new_assume_role_policy_document'] = policy
        if params:
            params['role_name'] = self.name
            return self.connection.update_role(**params)
        return False


class Policy(TaggedRAMObject):
    def __init__(self, connection=None):
        super(Policy, self).__init__(connection)

    def __repr__(self):
        return 'Policy:%s' % self.id

    def __getattr__(self, name):
        if name == 'name':
            return self.policy_name

    def __setattr__(self, name, value):
        if name == 'policy_name':
            self.name = value
        super(TaggedRAMObject, self).__setattr__(name, value)

    def read(self):
        policy = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            if name == 'policy':
                for k, v in value.items():
                    policy[k] = v
                continue
            if name == 'policy_name':
                policy['name'] = value
            # if name == 'policies':
            #     for k, v in value.items():
            #         policy[k] = v
            #     continue
            policy[name] = value
        return policy

    # def get(self):
    #     return self.connection.get_role(role_name=self.name)

    def delete(self):
        return self.connection.delete_policy(policy_name=self.name)

    def attach_policy_to_user(self, user_name=None, policy_type=None):
        policy_user = self.connection.list_policies_for_user(user_name=user_name)
        params = {'policy_name': self.name}
        attach = True
        for policy in policy_user:
            if policy.name == self.name:
                attach = False
        if attach:
            params['user_name'] = user_name
            params['policy_type'] = policy_type
            return self.connection.attach_policy_to_user(**params)
        return False

    def attach_policy_to_group(self, group_name=None, policy_type=None):
        policy_group = self.connection.list_policies_for_group(group_name=group_name)
        params = {'policy_name': self.name}
        attach = True
        for policy in policy_group:
            if policy.name == self.name:
                attach = False
        if attach:
            params['group_name'] = group_name
            params['policy_type'] = policy_type
            return self.connection.attach_policy_to_group(**params)
        return False

    def attach_policy_to_role(self, role_name=None, policy_type=None):
        policy_role = self.connection.list_policies_for_role(role_name=role_name)
        params = {'policy_name': self.name}
        attach = True
        for policy in policy_role:
            if policy.name == self.name:
                attach = False
        if attach:
            params['role_name'] = role_name
            params['policy_type'] = policy_type
            return self.connection.attach_policy_to_role(**params)
        return False

    def detach_policy_from_user(self, user_name=None, policy_type=None):
        policy_user = self.connection.list_policies_for_user(user_name=user_name)
        params = {'policy_name': self.name}
        detach = False
        for policy in policy_user:
            if policy.name == self.name:
                detach = True
        if detach:
            params['user_name'] = user_name
            params['policy_type'] = policy_type
            return self.connection.detach_policy_from_user(**params)
        return False

    def detach_policy_from_group(self, group_name=None, policy_type=None):
        policy_group = self.connection.list_policies_for_group(group_name=group_name)
        params = {'policy_name': self.name}
        detach = False
        for policy in policy_group:
            if policy.name == self.name:
                detach = True
        if detach:
            params['group_name'] = group_name
            params['policy_type'] = policy_type
            return self.connection.detach_policy_from_group(**params)
        return False

    def detach_policy_from_role(self, role_name=None, policy_type=None):
        policy_role = self.connection.list_policies_for_role(role_name=role_name)
        params = {'policy_name': self.name}
        detach = False
        for policy in policy_role:
            if policy.name == self.name:
                detach = True
        if detach:
            params['role_name'] = role_name
            params['policy_type'] = policy_type
            return self.connection.detach_policy_from_role(**params)
        return False



# encoding: utf-8
import time
from footmark.connection import ACSQueryConnection
from footmark.dns.regioninfo import RegionInfo
from footmark.exception import DNSResponseError
from footmark.resultset import ResultSet
from footmark.dns.dns import Dns
from footmark.dns.group import Group
from footmark.dns.config import *
from aliyunsdkcore.acs_exception.exceptions import ServerException
# from aliyunsdkalidns.request.v20150109.AddDomainRequest import


class DNSConnection(ACSQueryConnection):
    SDKVersion = '2015-01-09'
    DefaultRegionId = 'cn-hangzhou'
    DefaultRegionName = '杭州'.encode("UTF-8")
    ResponseError = DNSResponseError

    def __init__(self, acs_access_key_id=None, acs_secret_access_key=None,
                 region=None, sdk_version=None, security_token=None, ecs_role_name=None, user_agent=None):
        """
        Init method to create a new connection to DNS.
        """
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionId)
        self.region = region
        if sdk_version:
            self.SDKVersion = sdk_version

        self.STSSDK = 'aliyunsdkalidns.request.v' + self.SDKVersion.replace('-', '')

        super(DNSConnection, self).__init__(acs_access_key_id=acs_access_key_id,
                                            acs_secret_access_key=acs_secret_access_key,
                                            security_token=security_token,
                                            region=self.region, product=self.STSSDK,
                                            user_agent=user_agent,
                                            ecs_role_name=ecs_role_name)

    def add_domain(self, **kwargs):
        domain_name = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ResultSet).domain_name
        return self.describe_domain_info(domain_name=domain_name)

    def delete_domain(self, **kwargs):
        retry = 5
        while retry:
            try:
                return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))
            except ServerException as e:
                if str(e.error_code) == "Forbbiden" or str(e.error_code).find("Dependency"):
                    time.sleep(5)
                    retry -= 1
                    continue
                raise e
        return False

    def describe_domains(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['Domains', Dns])

    def describe_domain_info(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Dns)

    def update_domain_remark(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def get_main_domain_name(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Dns)

    def describe_domain_logs(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['Domains', Dns])

    def add_domain_group(self, **kwargs):
        group_id = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ResultSet).group_id
        kwargs['group_id'] = group_id
        return self.describe_domain_group(**kwargs)

    def update_domain_group(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def delete_domain_group(self, **kwargs):
        retry = 5
        while retry:
            try:
                return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))
            except ServerException as e:
                if str(e.error_code) == "Forbbiden" or str(e.error_code).find("Dependency"):
                    time.sleep(5)
                    retry -= 1
                    continue
                raise e
        return False

    def change_domain_group(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def describe_domain_groups(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['DomainGroups', Group])

    def describe_domain_group(self, **kwargs):
        groups = self.describe_domain_groups(**kwargs)
        match = ''
        for group in groups:
            if group.group_name == kwargs['group_name']:
                match = group
        return match



# encoding: utf-8
import json
from footmark.connection import ACSQueryConnection
from footmark.sts.regioninfo import RegionInfo
from footmark.exception import STSResponseError
from footmark.sts.sts import Sts
# from aliyunsdksts.request.v20150401.AssumeRoleRequest import


class STSConnection(ACSQueryConnection):
    SDKVersion = '2015-04-01'
    DefaultRegionId = 'cn-hangzhou'
    DefaultRegionName = '杭州'.encode("UTF-8")
    ResponseError = STSResponseError

    def __init__(self, acs_access_key_id=None, acs_secret_access_key=None,
                 region=None, sdk_version=None, security_token=None, ecs_role_name=None, user_agent=None):
        """
        Init method to create a new connection to STS.
        """
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionId)
        self.region = region
        if sdk_version:
            self.SDKVersion = sdk_version

        self.STSSDK = 'aliyunsdksts.request.v' + self.SDKVersion.replace('-', '')

        super(STSConnection, self).__init__(acs_access_key_id=acs_access_key_id,
                                            acs_secret_access_key=acs_secret_access_key,
                                            security_token=security_token,
                                            region=self.region, product=self.STSSDK,
                                            user_agent=user_agent,
                                            ecs_role_name=ecs_role_name)

    def format_sts_request_kwargs(self, **kwargs):
        for key, value in list(kwargs.items()):
            # format str to json
            if key == 'policy' and value:
                kwargs[key] = json.dumps(eval(value))
        return kwargs

    def assume_role(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_sts_request_kwargs(**self.format_request_kwargs(**kwargs))), Sts)

    def get_caller_identity(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_sts_request_kwargs(**self.format_request_kwargs(**kwargs))), Sts)



# encoding: utf-8
from footmark.connection import ACSQueryConnection
from footmark.oos.regioninfo import RegionInfo
from footmark.exception import OOSResponseError
from footmark.oos.oos import Template, Executions
# from aliyunsdkoos.request.v20190601.CreateTemplateRequest import


class OOSConnection(ACSQueryConnection):
    SDKVersion = '2019-06-01'
    DefaultRegionId = 'cn-hangzhou'
    DefaultRegionName = '杭州'.encode("UTF-8")
    ResponseError = OOSResponseError

    def __init__(self, acs_access_key_id=None, acs_secret_access_key=None,
                 region=None, sdk_version=None, security_token=None, ecs_role_name=None, user_agent=None):
        """
        Init method to create a new connection to OOS.
        """
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionId)
        self.region = region
        if sdk_version:
            self.SDKVersion = sdk_version

        self.RAMSDK = 'aliyunsdkoos.request.v' + self.SDKVersion.replace('-', '')

        super(OOSConnection, self).__init__(acs_access_key_id=acs_access_key_id,
                                            acs_secret_access_key=acs_secret_access_key,
                                            security_token=security_token,
                                            region=self.region, product=self.RAMSDK,
                                            user_agent=user_agent,
                                            ecs_role_name=ecs_role_name)

    def get_template(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Template)

    def delete_template(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def update_template(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Template)

    def list_templates(self, **kwargs):
        tmp = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Template).read()['templates']
        res = []
        if tmp:
            for t in tmp:
                elem = Template(self)
                for key, value in list(t.items()):
                    self.parse_value(value)
                    setattr(elem, self.convert_name(key), value)
                res.append(elem)
        return res

    def create_template(self, **kwargs):
        tags = kwargs.get('tags', '')
        kwargs = self.format_request_kwargs(**kwargs)
        if tags:
            kwargs['tags'] = tags
        else:
            kwargs.pop('tags')
        return self.get_object_new(self.build_request_params(kwargs), Template)

    def list_execution_risky_tasks(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Template)

    def start_execution(self, **kwargs):
        tags = kwargs.get('tags', '')
        kwargs = self.format_request_kwargs(**kwargs)
        if tags:
            kwargs['tags'] = tags
        else:
            kwargs.pop('tags')
        return self.get_object_new(self.build_request_params(kwargs), Executions)

    def cancel_execution(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def delete_executions(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def list_executions(self, **kwargs):
        tmp = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Executions).read()['executions']
        res = []
        if tmp:
            for t in tmp:
                elem = Executions(self)
                for key, value in list(t.items()):
                    self.parse_value(value)
                    setattr(elem, self.convert_name(key), value)
                res.append(elem)
        return res

    def notify_execution(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def list_tag_resources(self, **kwargs):
        res = {}
        tags = kwargs['tags']
        kwargs = self.format_request_kwargs(**kwargs)
        kwargs['tags'] = tags
        tags = self.get_list_new(self.build_request_params(kwargs), ['TagResources', Template])
        for tag in tags:
            res[tag.tag_key] = tag.tag_value
        return res

    def tag_resources(self, **kwargs):
        tmp = {}
        if kwargs['tags']:
            for key, value in list(kwargs['tags'].items()):
                if key in list(self.list_tag_resources(**kwargs).keys()) and value == self.list_tag_resources(**kwargs)[key]:
                    continue
                tmp[key] = value
        if tmp:
            kwargs = self.format_request_kwargs(**kwargs)
            kwargs['tags'] = tmp
            return self.get_status_new(self.build_request_params(kwargs))
        return None

    def untag_resources(self, **kwargs):
        tmp = []
        if kwargs['tags']:
            for key, value in list(kwargs['tags'].items()):
                if key not in list(self.list_tag_resources(**kwargs).keys()):
                    continue
                tmp.append(key)
        if tmp:
            kwargs['tag_keys'] = tmp
            return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))
        return False



from footmark.ros.connection import ROSConnection


def connect_to_region(region_id, **kw_params):
    """
    Given a valid region name, return a
    :class:`footmark.ros.connection.ROSConnection`.
    Any additional parameters after the region_name are passed on to
    the connect method of the region object.

    :type: str
    :param region_id: The ID of the region to connect to.

    :rtype: :class:`footmark.ros.connection.ROSConnection` or ``None``
    :return: A connection to the given region, or None if an invalid region
             name is given
    """
    return ROSConnection(region=region_id, **kw_params)



from footmark.market.productobject import TaggedPRODUCTObject


class Product(TaggedPRODUCTObject):
    def __init__(self, connection=None):
        super(Product, self).__init__(connection)

    def __repr__(self):
        return 'Product:%s' % self.id

    def __getattr__(self, name):
        if name == 'price':
            return self.suggested_price

    def __setattr__(self, name, value):
        super(TaggedPRODUCTObject, self).__setattr__(name, value)

    def get(self):
        return self.connection.describe_product(code=self.code)

    def read(self):
        product = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id", "description", "product_extras","shop_info", "pic_url"]:
                continue
            if name == 'product_skus':
                for m in value['product_sku'][0]['modules']['module']:
                    if m['code'] == 'img_id':
                        res = m['properties']['property'][0]['property_values']['property_value']
                product['image_ids'] = res
                continue
            product[name] = value
        return product

    def read_with_region_name(self, region_name):
        product = {}

        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id", "description", "product_extras","shop_info", "pic_url"]:
                continue
            if name == 'product_skus':
                result = []
                for m in value['product_sku'][0]['modules']['module']:
                    if m['code'] == 'img_id':
                        res = m['properties']['property'][0]['property_values']['property_value']
                        for r in res:
                            if r['display_name'].startswith(region_name):
                                result.append(r)
                product['images'] = result
                continue
            product[name] = value
        return product



# encoding: utf-8
import json
from footmark.connection import ACSQueryConnection
from footmark.ram.regioninfo import RegionInfo
from footmark.exception import RAMResponseError
from footmark.ram.ram import User, Profile, Access, Group, Role, Policy
# from aliyunsdkram.request.v20150501.CreateRoleRequest import


class RAMConnection(ACSQueryConnection):
    SDKVersion = '2015-05-01'
    DefaultRegionId = 'cn-hangzhou'
    DefaultRegionName = '杭州'.encode("UTF-8")
    ResponseError = RAMResponseError

    def __init__(self, acs_access_key_id=None, acs_secret_access_key=None,
                 region=None, sdk_version=None, security_token=None, ecs_role_name=None, user_agent=None):
        """
        Init method to create a new connection to STS.
        """
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionId)
        self.region = region
        if sdk_version:
            self.SDKVersion = sdk_version

        self.RAMSDK = 'aliyunsdkram.request.v' + self.SDKVersion.replace('-', '')

        super(RAMConnection, self).__init__(acs_access_key_id=acs_access_key_id,
                                            acs_secret_access_key=acs_secret_access_key,
                                            security_token=security_token,
                                            region=self.region, product=self.RAMSDK,
                                            user_agent=user_agent,
                                            ecs_role_name=ecs_role_name)

    def list_users(self, **kwargs):
        results = []
        users = []
        is_truncated = True
        while is_truncated:
            res = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), User)
            users.extend(res.users['user'])
            is_truncated = res.is_truncated
            kwargs['marker'] = res.marker

        for user in users:
            element = User(self)
            for k, v in list(user.items()):
                setattr(element, k, v)
            results.append(element)
        return results

    def delete_user(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def create_user(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), User)

    def update_user(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), User)

    def get_login_profile(self, **kwargs):
        try:
            return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Profile)
        except:
            return None

    def create_login_profile(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Profile)

    def delete_login_profile(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def update_login_profile(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def create_access_key(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Access)

    def list_access_keys(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['AccessKeys', Access])

    def update_access_key(self, **kwargs):
        if self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs))):
            access_keys = self.list_access_keys(**kwargs)
            for access in access_keys:
                if kwargs['user_access_key_id'] == access.read()['access_key_id']:
                    return access
        return None

    def delete_access_key(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def create_group(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Group)

    def update_group(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def delete_group(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def list_groups(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['Groups', Group])

    def add_user_to_group(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def remove_user_from_group(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def list_users_for_group(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['Users', Group])

    def create_role(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Role)

    def list_roles(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['Roles', Role])

    def get_role(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Role)

    def update_role(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def delete_role(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def create_policy(self, **kwargs):
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Policy)

    def delete_policy(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def get_policy(self, **kwargs):
        if not kwargs.get('policy_type'):
            kwargs['policy_type'] = 'Custom'
        return self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Policy)

    def attach_policy_to_user(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def attach_policy_to_group(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def attach_policy_to_role(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def detach_policy_from_user(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def detach_policy_from_group(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def detach_policy_from_role(self, **kwargs):
        return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))

    def list_policies(self, **kwargs):
        results = []
        policies = []
        is_truncated = True
        while is_truncated:
            res = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), Policy)
            policies.extend(res.policies['policy'])
            is_truncated = res.is_truncated
            kwargs['marker'] = res.marker

        for policy in policies:
            element = Policy(self)
            for k, v in list(policy.items()):
                setattr(element, k, v)
            results.append(element)
        return results

    def list_policies_for_user(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['Policies', Policy])

    def list_policies_for_group(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['Policies', Policy])

    def list_policies_for_role(self, **kwargs):
        return self.get_list_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ['Policies', Policy])




"""
Represents an ECS key pair
"""
from footmark.ecs.ecsobject import TaggedECSObject


class KeyPair(TaggedECSObject):
    """
    Represents an ECS key pair.

    :ivar id: The unique ID of the key pair.
    :ivar name: The name of the key pair.
    :ivar create_time: The timestamp of when the key pair was created.
    :ivar finger_print: The finger print of the key pair, see RFC4716.
    :ivar private_key: The private key of key pair in format PEM PKCS#8.
    """

    def __init__(self, connection=None):
        super(KeyPair, self).__init__(connection)
        self.tags = {}

    def __repr__(self):
        return 'KeyPair:%s' % self.id

    def __getattr__(self, name):
        if name == 'id':
            return self.key_pair_id
        if name == 'name':
            return self.key_pair_name
        if name == 'finger_print':
            return self.key_pair_finger_print
        raise AttributeError

    def __setattr__(self, name, value):
        if name == 'id':
            self.key_pair_id = value
        if name == 'name':
            self.key_pair_name = value
        if name == 'tags' and value:
            v = {}
            for tag in value['tag']:
                v[tag.get('TagKey')] = tag.get('TagValue', None)
            value = v
        super(TaggedECSObject, self).__setattr__(name, value)

    def import_key(self, name=None, public_key_body=None):
        """
        Import a key pair
        """
        params = {}

        if name and self.key_pair_name != name:
            params['key_pair_name'] = name
        if public_key_body:
            params['public_key_body'] = public_key_body
        if params:
            return self.connection.import_key_pair(**params)

        return False

    def delete(self):
        return self.connection.delete_key_pair(key_pair_names=[self.name])

    def get(self):
        return self.connection.describe_key_pairs(key_pair_name=self.name)

    def read(self):
        key_pairs = {}

        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region"]:
                continue
            if name == 'key_pair_id':
                key_pairs['id'] = value
            if name == 'key_pair_name':
                key_pairs['name'] = value
            if name == 'finger_print':
                key_pairs['finger_print'] = value
            key_pairs[name] = value
        return key_pairs



import time
from footmark.connection import ACSQueryConnection
from footmark.exception import ROSResponseError
from footmark.resultset import ResultSet
from aliyunsdkcore.acs_exception.exceptions import ServerException


class ROSConnection(ACSQueryConnection):
    SDKVersion = '2019-09-10'
    DefaultRegionId = 'cn-hangzhou'
    DefaultRegionName = '杭州'.encode("UTF-8")

    ResponseError = ROSResponseError

    def __init__(self, acs_access_key_id=None, acs_secret_access_key=None,
                 region=None, sdk_version=None, security_token=None, user_agent=None, ecs_role_name=None):
        """
        Init method to create a new connection to ROS.
        """
        if not region:
            region = self.DefaultRegionId
        self.region = region
        if sdk_version:
            self.SDKVersion = sdk_version

        self.ROSSDK = 'aliyunsdkros.request.v' + self.SDKVersion.replace('-', '')

        super(ROSConnection, self).__init__(acs_access_key_id=acs_access_key_id,
                                            acs_secret_access_key=acs_secret_access_key,
                                            security_token=security_token,
                                            region=self.region, product=self.ROSSDK,
                                            user_agent=user_agent, ecs_role_name=ecs_role_name)

    def create_stack(self, **kwargs):
        stack_id = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)),
                                       ResultSet).stack_id
        self.wait_for_ros_status(stack_id, 'CREATE_COMPLETE', 4, 720)
        return self.get_stack(stack_id=stack_id)

    def delete_stack(self, **kwargs):
        retry = 5
        while retry:
            try:
                return self.get_status_new(self.build_request_params(self.format_request_kwargs(**kwargs)))
            except ServerException as e:
                if str(e.error_code) == "Forbbiden" or str(e.error_code).find("Dependency"):
                    time.sleep(5)
                    retry -= 1
                    continue
        return False

    def update_stack(self, **kwargs):
        try:
            stack_id = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)),
                                           ResultSet).stack_id
            self.wait_for_ros_status(stack_id, 'UPDATE_COMPLETE', 4, 720)
            return self.get_stack(stack_id=stack_id)
        except Exception as e:
            raise ROSResponseError(e)

    def list_stacks(self, **kwargs):
        ros_stack_objs = self.get_object_new(self.build_request_params(self.format_request_kwargs(**kwargs)), ResultSet)
        if hasattr(ros_stack_objs, 'stacks') and ros_stack_objs.stacks:
            if kwargs.get('get_one'):
                return ros_stack_objs.stacks[0]
            else:
                return ros_stack_objs.stacks
        else:
            return []

    def get_stack(self, stack_id):
        ros_stack_obj = self.get_object_new(self.build_request_params(self.format_request_kwargs(stack_id=stack_id)),
                                            ResultSet)

        if ros_stack_obj:
            return {'status': ros_stack_obj.status,
                    'status_reason': ros_stack_obj.status_reason,
                    'create_time': ros_stack_obj.create_time,
                    'region_id': ros_stack_obj.region_id,
                    'disable_rollback': ros_stack_obj.disable_rollback,
                    'stack_name': ros_stack_obj.stack_name,
                    'stack_id': ros_stack_obj.stack_id,
                    'timeout_in_minutes': ros_stack_obj.timeout_in_minutes,
                    'outputs': ros_stack_obj.outputs if hasattr(ros_stack_obj, 'outputs') else []}
        else:
            return {}

    def wait_for_ros_status(self, stack_id, status, delay, timeout):

        try:
            while True:
                stack = self.get_stack(stack_id=stack_id)
                if stack and str(stack['status']) in [status, str(status).lower()]:
                    return True

                timeout -= delay

                if timeout <= 0:
                    raise Exception("Timeout Error: Waiting for ROS stack status is %s, time-consuming %d seconds." % (status, timeout))

                time.sleep(delay)
        except Exception as e:
            raise e




from footmark.rds.rdsobject import TaggedRDSObject


class DbInstance(TaggedRDSObject):
    def __init__(self, connection=None):
        super(DbInstance, self).__init__(connection)

    def __repr__(self):
        return 'DbInstance:%s' % self.id

    def __getattr__(self, name):
        if name == 'id':
            return self.dbinstance_id
        if name == 'status':
            return self.dbinstance_status
        if name == 'type':
            return self.dbinstance_type
        if name == 'connection_string':
            return self.connection_string
        if name == 'name':
            return self.dbinstance_description

    def __setattr__(self, name, value):
        if name == 'dbinstance_id':
            self.id = value
        if name == 'dbinstance_status':
            self.status = value
        if name == 'dbinstance_type':
            self.type = value
        if name == 'dbinstance_description':
            self.name = value
        super(TaggedRDSObject, self).__setattr__(name, value)

    def get(self):
        return self.connection.describe_db_instance_attribute(db_instance_id=self.id)

    def read(self):
        ins = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "order_id"]:
                continue
            if name in ['dbinstance_class', 'dbinstance_description', 'dbinstance_id', 'dbinstance_net_type', 'dbinstance_status', 'dbinstance_type', 'dbinstance_class_type', 'dbinstance_cpu', 'dbinstance_memory', 'dbinstance_storage', 'dbinstance_storage_type', 'dbmax_quantity']:
                new_name = name.replace('db', 'db_')
                ins[new_name] = value
                continue
            ins[name] = value
        return ins

    def restart(self):
        return self.connection.restart_db_instance(db_instance_id=self.id)

    def delete(self):
        return self.connection.delete_db_instance(db_instance_id=self.id)

    def allocate_public_connection_string(self, connection_string_prefix=None, port=None):
        params = {}
        if connection_string_prefix and port:
            params['connection_string_prefix'] = connection_string_prefix
            params['port'] = port
            params['db_instance_id'] = self.id
            return self.connection.allocate_instance_public_connection(**params)

    def modify_db_instance_connection_string(self, connection_string_prefix=None, port=None, current_connection_string=None):
        info_obj_list = self.connection.describe_db_instance_net_info(dbinstance_id=self.id)
        res = ''
        for obj in info_obj_list:
            if obj.connection_string == current_connection_string and not obj.connection_string.startswith(connection_string_prefix):
                res = obj
        params = {}
        if res:
            params['connection_string_prefix'] = connection_string_prefix
            params['port'] = port
            params['current_connection_string'] = current_connection_string
            params['db_instance_id'] = self.id
            return self.connection.modify_db_instance_connection_string(**params)
        return False

    def modify_network_type(self, instance_network_type=None):
        params = {}
        if instance_network_type and self.instance_network_type != instance_network_type:
            params['instance_network_type'] = instance_network_type
            params['db_instance_id'] = self.id
            return self.connection.modify_db_instance_network_type(**params)
        return False

    def modify_description(self, db_instance_description=None):
        params = {}
        if db_instance_description and db_instance_description != self.name:
            params['db_instance_description'] = db_instance_description
            params['db_instance_id'] = self.id
            return self.connection.modify_db_instance_description(**params)
        return False

    def modify_instance_spec(self, db_instance_class=None, db_instance_storage=None):
        params = {}
        params['db_instance_id'] = self.id
        params['pay_type'] = self.pay_type
        if db_instance_class and self.db_instance_class != db_instance_class:
            params['db_instance_class'] = db_instance_class
        if db_instance_storage and self.db_instance_storage != db_instance_storage:
            params['db_instance_storage'] = db_instance_storage
        if params.get('db_instance_class') or params.get('db_instance_storage'):
            return self.connection.modify_db_instance_spec(**params)
        else:
            return False

    def list_tags(self):
        res = {}
        tags = self.connection.list_tag_resources(resource_ids=[self.id], resource_type='instance')
        for tag in tags:
            res[tag.tag_key] = tag.tag_value
        return res

    def add_tags(self, tags):
        """
        Add tags
        """
        tmp = {}
        if tags:
            for key, value in list(tags.items()):
                if key in list(self.list_tags().keys()) and value == self.list_tags()[key]:
                    continue
                tmp[key] = value
        if tmp:
            return self.connection.tag_resources(resource_ids=[self.id], tags=tmp, resource_type='instance')
        return False

    def remove_tags(self, tags):
        """
        remove tags
        """
        tmp = []
        if tags:
            for key, value in list(tags.items()):
                if key not in list(self.list_tags().keys()):
                    continue
                tmp.append(key)
        if tmp:
            return self.connection.untag_resources(resource_ids=[self.id], tag_keys=tmp, resource_type='instance')
        return False


class Database(TaggedRDSObject):
    def __init__(self, connection=None):
        super(Database, self).__init__(connection)

    def __repr__(self):
        return 'Database:%s' % self.id

    def __getattr__(self, name):
        if name == 'id':
            return self.dbinstance_id
        if name == 'status':
            return self.dbstatus
        if name == 'description':
            return self.dbdescription
        if name == 'name':
            return self.dbname

    def __setattr__(self, name, value):
        if name == 'dbinstance_id':
            self.id = value
        if name == 'dbname':
            self.name = value
        if name == 'dbdescription':
            self.description = value
        if name == 'dbstatus':
            self.status = value
        super(TaggedRDSObject, self).__setattr__(name, value)

    def get(self):
        return self.connection.describe_databases(db_instance_id=self.id)[0]

    def read(self):
        db = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region"]:
                continue
            if name in ['dbdescription', 'dbinstance_id', 'dbname', 'dbstatus']:
                new_name = name.replace('db', 'db_')
                db[new_name] = value
                continue
            db[name] = value
        return db

    def delete(self):
        return self.connection.delete_database(db_instance_id=self.id, db_name=self.name)

    def modify_db_description(self, description=None):
        params = {}
        if description and self.description != description:
            params['DBDescription'] = description
            params['DBName'] = self.name
            params['DBInstanceId'] = self.id
            return self.connection.modify_db_description(**params)
        return False

    def copy_database_between_instances(self, **kwargs):
        target_instance_id = kwargs['target_db_instance_id']
        target_name = kwargs['target_db_name']
        target_instance_db = self.connection.describe_databases(db_instance_id=target_instance_id)
        for db in target_instance_db:
            if db.name == target_name:
                return False
        return self.connection.copy_database_between_instances(**kwargs)


class Account(TaggedRDSObject):
    def __init__(self, connection=None, owner_id=None,
                 name=None, description=None, id=None):
        super(Account, self).__init__(connection)
        self.tags = {}

    def __repr__(self):
        return 'Account:%s' % self.name

    def __getattr__(self, name):
        if name == 'name':
            return self.account_name
        if name == 'status':
            return self.account_status
        if name == 'description':
            return self.account_description
        if name == 'privileges':
            return self.database_privileges
        if name == 'type':
            return self.account_type
        if name == 'id':
            return self.dbinstance_id

    def __setattr__(self, name, value):
        if name == 'account_name':
            self.name = value
        if name == 'account_status':
            self.status = value
        if name == 'database_privileges':
            self.privileges = value
        if name == 'account_type':
            self.type = value
        super(TaggedRDSObject, self).__setattr__(name, value)

    def grant_privilege(self, db_names, account_privilege):
        '''
        grant privilege
        '''
        flag = False
        for db_name in db_names:
            flag = self.connection.grant_account_privilege(db_instance_id=self.id, account_name=self.name, db_name=db_name, account_privilege=account_privilege)
        return flag

    def revoke_privilege(self, db_names):
        '''
        revoke privilege
        '''
        flag = False
        for db_name in db_names:
            flag = self.connection.revoke_account_privilege(db_instance_id=self.id, account_name=self.name, db_name=db_name)
        return flag

    def modify_description(self, description=None):
        '''
        modify description
        '''
        params = {}
        if description:
            params['db_instance_id'] = self.id
            params['account_name'] = self.name
            params['account_description'] = description
        return self.connection.modify_account_description(**params)
    
    def delete(self):
        '''
        delete account 
        '''
        return self.connection.delete_account(db_instance_id=self.id, account_name=self.name)

    def reset(self, account_password):
        return self.connection.reset_account_password(account_password=account_password, db_instance_id=self.id, account_name=self.name)

    def read(self):
        account = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region"]:
                continue
            if name == 'dbinstance_id':
                account['db_instance_id'] = value
                continue
            account[name] = value
        return account

    def get(self):
        return self.connection.describe_accounts(db_instance_id=self.id, account_name=self.name)[0]


class BackUp(TaggedRDSObject):
    def __init__(self, connection=None, owner_id=None,
                 name=None, description=None, id=None):
        super(BackUp, self).__init__(connection)
        self.tags = {}

    def __repr__(self):
        return 'BackUp:%s' % self.name

    def __getattr__(self, name):
        if name == 'id':
            return self.backup_id
        if name == 'instance_id':
            return self.dbinstance_id
        if name == 'status':
            return self.backup_status
        if name == 'mode':
            return self.backup_mode

    def __setattr__(self, name, value):
        if name == 'backup_id':
            self.id = value
        if name == 'dbinstance_id':
            self.instance_id = value
        if name == 'backup_status':
            self.status = value
        if name == 'backup_mode':
            self.mode = value
        super(TaggedRDSObject, self).__setattr__(name, value)

    def read(self):
        backup = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region"]:
                continue
            if name == 'dbinstance_id':
                backup['db_instance_id'] = value
                continue
            backup[name] = value
        return backup




from footmark.regioninfo import RegionInfo


class MARKETRegionInfo(RegionInfo):
    """
    Represents an MARKET Region
    """

    def __init__(self, connection=None, name=None, id=None,
                 connection_cls=None):
        from footmark.market.connection import MARKETConnection
        super(MARKETRegionInfo, self).__init__(connection, name, id,
                                               MARKETConnection)




"""
Represents an VPC Security Group
"""
from footmark.sts.stsobject import TaggedSTSObject


class Sts(TaggedSTSObject):
    def __init__(self, connection=None):
        super(Sts, self).__init__(connection)

    def __repr__(self):
        return 'Ram:%s' % self.id

    def __getattr__(self, name):
        if name == 'AccessKeyId':
            return self.access_key_id
        if name == 'AccessKeySecret':
            return self.access_key_secret
        if name == 'SecurityToken':
            return self.security_token

    def __setattr__(self, name, value):
        super(TaggedSTSObject, self).__setattr__(name, value)

    def read(self):
        ram = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            if name in ['credentials', 'assumed_role_user']:
                for key, value in value.items():
                    ram[key] = value
            ram[name] = value
        return ram




from footmark.dns.dnsobject import TaggedDNSObject


class Group(TaggedDNSObject):
    def __init__(self, connection=None):
        super(Group, self).__init__(connection)

    def __repr__(self):
        return 'Group:%s' % self.id

    def __getattr__(self, name):
        if name == 'id':
            return self.id
        if name == 'name':
            return self.group_name
        if name == 'domain_count':
            return self.count
        raise AttributeError

    def __setattr__(self, name, value):
        if name == 'group_id':
            self.id = value
        if name == 'domain_count':
            self.count = value
        super(TaggedDNSObject, self).__setattr__(name, value)

    def get(self):
        return self.connection.describe_domain_group(group_id=self.id, group_name=self.name)

    def read(self):
        group = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region"]:
                continue
            if name == 'group_id':
                group['id'] = value
            group[name] = value
        return group

    def update_domain_group(self, new_group_name=None, group_name=None):
        group_id = self.connection.describe_domain_group(group_name=group_name).id
        if new_group_name and group_name:
            return self.connection.update_domain_group(group_id=group_id, group_name=new_group_name)
        return False

    def delete(self):
        return self.connection.delete_domain_group(group_id=self.id)



from footmark.oos.oosobject import TaggedOOSObject


class Template(TaggedOOSObject):
    def __init__(self, connection=None):
        super(Template, self).__init__(connection)

    def __repr__(self):
        return 'Template:%s' % self.id

    def __getattr__(self, name):
        if name == 'name':
            return self.template_name
        if name == 'id':
            return self.template_id
        if name == 'version':
            return self.template_version

    def __setattr__(self, name, value):
        if name == 'template_name':
            self.name = value
        if name == 'template_id':
            self.id = value
        if name == 'template_version':
            self.version = value
        if name == 'template':
            for k, v in value.items():
                if k == 'template_id':
                    self.id = v
                if k == 'template_name':
                    self.name = v
                if k == 'template_version':
                    self.version = v
        super(TaggedOOSObject, self).__setattr__(name, value)

    def risk(self):
        return self.connection.list_execution_risky_tasks(template_name=self.name)

    def read(self):
        OOS = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            if name == 'template':
                for k, v in value.items():
                    OOS[k] = v
                continue
            OOS[name] = value
        return OOS

    def delete(self):
        return self.connection.delete_template(template_name=self.name)

    def update(self, content):
        if content != self.content:
            return self.connection.update_template(template_name=self.name, content=content)
        return None

    def get(self):
        return self.connection.get_template(template_name=self.name)

    def add_tags(self, tags):
        """
        Add tags
        """
        return self.connection.tag_resources(resource_ids=[self.name], tags=tags, resource_type='template')

    def remove_tags(self, tags):
        """
        remove tags
        """
        return self.connection.untag_resources(resource_ids=[self.name], tags=tags, resource_type='template')


class Executions(TaggedOOSObject):
    def __init__(self, connection=None):
        super(Executions, self).__init__(connection)

    def __repr__(self):
        return 'Executions:%s' % self.id

    def __getattr__(self, name):
        if name == 'name':
            return self.template_name
        if name == 'id':
            return self.execution_id

    def __setattr__(self, name, value):
        if name == 'execution':
            for k, v in value.items():
                if k == 'execution_id':
                    self.id = v
                if k == 'template_name':
                    self.name = v
        super(TaggedOOSObject, self).__setattr__(name, value)

    def read(self):
        execution = {}
        for name, value in list(self.__dict__.items()):
            if name in ["connection", "region_id", "region", "request_id"]:
                continue
            if name == 'execution':
                for k, v in value.items():
                    execution[k] = v
                continue
            execution[name] = value
        return execution
